#!/bin/bash

. ./bootstrap-env

INSTANCE_ID=$(curl http://169.254.169.254/latest/meta-data/instance-id)
LB_NAME=$(aws s3 cp s3://$STATE_BUCKET/apps/$APP_NAME/lb-name -)

cat <<EOF | logger -t bootstrap
  beginning instance bootstrap for $INSTANCE_ID
    app: $APP_NAME
    state_bucket: $STATE_BUCKET
    lb_name: $LB_NAME
    fleet_name: $FLEET_NAME
EOF

if [ $FLEET_NAME == "web" ]; then
  echo "Registering with load balancer $LB_NAME" | logger -t bootstrap

  aws elb register-instances-with-load-balancer \
    --load-balancer-name $LB_NAME \
    --instances $INSTANCE_ID \
    --region $AWS_REGION
fi

mkdir -p /var/app

cat > /var/app/env <<EOF
APP_NAME=$APP_NAME
STATE_BUCKET=$STATE_BUCKET
FLEET_NAME=$FLEET_NAME
EOF

echo "Wrote /var/app/env" | logger -t bootstrap

chown -R ubuntu:ubuntu /var/app

mkdir -p /home/ubuntu/.ssh
echo "$JENKINS_PUBLIC_KEY" >> /home/ubuntu/.ssh/authorized_keys
chown -R ubuntu:ubuntu /home/ubuntu/.ssh

echo "Wrote jenkins public key" | logger -t bootstrap

cat > /usr/local/bin/deploy-app <<EOF
#!/bin/bash

cd /var/app
. ./env

DEPLOYED_VERSION=`cat /var/app/release_id`
VERSION=\$1

if [ -z \$VERSION ]; then
  VERSION=\$(aws s3 cp s3://\$STATE_BUCKET/apps/\$APP_NAME/current_release_id -)
fi

if [ "\$VERSION" == "\$DEPLOYED_VERSION" ]; then
  echo "The desired version (\$VERSION) is currently deployed; exiting"
  exit 0
elif [ -z \$DEPLOYED_VERSION ]; then
  echo "Bootstrapping instance with version \$VERSION"
else
  echo "Had version \$DEPLOYED_VERSION; deploying \$VERSION"
fi

aws s3 cp s3://\$STATE_BUCKET/apps/\$APP_NAME/releases/v\$VERSION.tgz /tmp/slug.tgz

rm -rf staging || true
mkdir staging
cd staging
tar -xzf /tmp/slug.tgz

. bin/pre-deploy

cd /var/app

rm -rf live
mv staging live

cd live

. bin/restart

echo \$VERSION > /var/app/release_id
EOF

cat > /usr/local/bin/restart-app <<EOF
#!/bin/bash

cd /var/app
. ./env

cd live

. bin/restart
EOF


cat > /usr/local/bin/fork-and-deploy-app <<EOF
#!/bin/bash -l

( setsid deploy-app 2>&1 | setsid logger -t deploy ) & disown

sleep 1
echo "Forked... bye"
EOF

cat > /usr/local/bin/fork-and-restart-app <<EOF
#!/bin/bash -l

( setsid restart-app 2>&1 | setsid logger -t deploy ) & disown

sleep 1
echo "Forked... bye"
EOF


chmod +x /usr/local/bin/deploy-app
chmod +x /usr/local/bin/fork-and-deploy-app
chmod +x /usr/local/bin/restart-app
chmod +x /usr/local/bin/fork-and-restart-app

echo "Wrote deploy scripts" | logger -t bootstrap

echo "Running initial deploy as user 'ubuntu'..."
nohup sudo -u ubuntu fork-and-deploy-app & disown

echo "That's it for bootstrap!"
