class Inf::Domains < Thor
  def method_missing(m, *args, &block)
    Inf.send(m, *args, &block)
  end

  def self.method_missing(m, *args, &block)
    Inf.send(m, *args, &block)
  end

  def self.list
    get_state("apps/#{app_name}/domains").lines rescue []
  end

  def self.hosted_zones
    # @hosted_zones ||= route53.list_hosted_zones_by_name.hosted_zones.reduce
    #   binding.pry
    # end
  end

  def self.sync_to_route53
    lb_dns = get_state("apps/#{app_name}/lb-dns")

    hosted_zones

    list.each do |domain|
      route53.change_resource_record_sets({
        change_batch: {
          changes: [
            {
              action: "UPSERT",
              resource_record_set: {
                name: domain,
                resource_records: [
                  {
                    value: lb_dns,
                  },
                ],
                ttl: ENV['DNS_TTL'],
                type: "CNAME",
              },
            },
          ],
          comment: "load balancer for #{app_name}",
        },
        hosted_zone_id: zone_id,
      })
    end
  end

  desc 'domains', 'list domains'
  def list
    Inf::Domains.list.each do |domain|
      puts domain
    end
  end

  desc 'sync', 'sync to route53'
  def sync
    Inf::Domains.sync_to_route53
  end

  desc 'add DOMAIN', 'cname DOMAIN to the load balancer'
  def add(domain)
    domains = Inf::Domains.list

    if domains.include? domain
      puts "Domain #{domain} is already configured".blue
      return
    end

    domains += [domain]
    put_state("apps/#{app_name}/domains", domains.join("\n"))

    Inf::Domains.sync_to_route53
  end

  desc 'remove DOMAIN', 'un-cname DOMAIN'
  def remove(domain)
    domains = Inf::Domains.list

    unless domains.include? domain
      puts "Domain #{domain} is not configured".red
      return
    end

    domains = domains.reject { |d| d == domain }
    put_state("apps/#{app_name}/domains", domains.join("\n"))

    Inf::Domains.sync_to_route53
  end
end
