require 'mini_cache'

module Inf
  def self.cache
    @cache ||= MiniCache::Store.new
  end

  def self.state_bucket
    ENV['STATE_BUCKET'].downcase
  end

  def self.app_name
    ENV['APP_NAME'].downcase
  end

  def self.put_state(k, v)
    v = v.to_s
    s3.put_object(
      bucket: state_bucket,
      key: k,
      body: v
    )

    summary = v.lines.count > 1 ? '[multiline]' : v
    cache.unset k
    puts "#{k} -> #{summary}".gray
  end

  def self.list_state(prefix)
    s3.list_objects_v2(
      bucket: state_bucket,
      prefix: prefix,
    ).contents.map do |it|
      yield it.key
    end
  end

  def self.get_state(k, opts={})
    cache.get_or_set(k) do
      begin
        object = s3.get_object(bucket: state_bucket, key: k)

        if opts[:raw]
          object
        else
          object.body.read
        end
      rescue Aws::S3::Errors::NoSuchKey
      end
    end
  end

  def self.delete_state(k)
    s3.delete_object(bucket: state_bucket, key: k) rescue nil
  end

  class << self
    %i(s3 ec2 elb route53).each do |m|
      define_method m do
        AWS.send m
      end
    end
  end
end

require 'thor'

require_relative 'version.rb'

require_relative 'migrations.rb'
require_relative 'aws.rb'
require_relative 'fleet.rb'
require_relative 'env.rb'
require_relative 'load_balancer.rb'
# require_relative 'domains.rb'
