class Inf::LoadBalancer < Thor
  def method_missing(m, *args, &block)
    Inf.send(m, *args, &block)
  end

  def self.method_missing(m, *args, &block)
    Inf.send(m, *args, &block)
  end

  desc 'launch', 'launch load balancer for app'
  def launch
    Inf::AWS.launch_load_balancer
  end

  desc 'kill', 'kill lb for app'
  def kill
    lb_name  = get_state "apps/#{app_name}/lb-name"
    delete_state "apps/#{app_name}/lb-name"

    if lb_name
      elb.delete_load_balancer(
        load_balancer_name: lb_name
      )

      delete_state "apps/#{app_name}/lb-dns"
      puts "Deleted lb #{lb_name}".blue
    else
      puts "Did not find an lb".yellow
    end
  end

  desc 'status', 'lb status'
  def status
    lb_name  = get_state "apps/#{app_name}/lb-name"

    unless lb_name
      puts "Load balancer is not configured".blue
      return
    end

    lb = elb.
      describe_load_balancers(load_balancer_names: [lb_name]).
      load_balancer_descriptions[0]

    unless lb
      puts "Load balancer expected but not found!".red
      return
    end

    puts "Load balancer is at #{lb.dns_name} with ".blue +
      "#{lb.instances.count} instances".blue
  end
end
